@@ -5,7 +5,7 @@ import fnmatch
 import os
 from datetime import datetime, timedelta
 from types import SimpleNamespace
-from typing import List, Optional, Tuple, Dict, Union
+from typing import List, Optional, Tuple, Dict, Union, Set
 
 from helix_catalog_sdk.data_source import DataSource, ResourceItem
 
@@ -80,6 +80,8 @@ class Catalog:
     ) -> None:
         """
         Updates the last_processed path with the passed in date using the date_format of the resource
+
+
         :param resource_name: resource name to update
         :param updated_date: date to update last_processed with
         :param data_source: data source to update
@@ -247,7 +249,7 @@ class Catalog:
         self, resource: ResourceItem, last_processed: str
     ) -> List[str]:
         full_path = resource.full_path
-        all_paths: List[str] = []
+        all_paths: Set[str] = set()
         if full_path.startswith("s3://"):
             path = full_path.replace("s3://", "")
             bucket = path.split("/")[0]
@@ -268,21 +270,35 @@ class Catalog:
                         or last_processed_file_name == s3_file_name
                         or fnmatch.fnmatch(s3_file_name, last_processed_file_name)
                     ):
-                        all_paths.append(f"s3://{bucket}/{s3_file['Key']}")
+                        if (
+                            resource.load_folders
+                        ):  # load at folder level instead of file level
+                            s3_folder_parts = s3_path_parts[0:-1]
+                            s3_folder = "/".join(s3_folder_parts)
+                            all_paths.add(f"s3://{bucket}/{s3_folder}")
+                        else:
+                            all_paths.add(f"s3://{bucket}/{s3_file['Key']}")
                     else:
                         self.logger.debug(
                             f"Did not match {s3_file['Key']} to {last_processed_file_name}"
                         )
-            return all_paths
+            return list(all_paths)
         else:
+            # handle non-S3 paths (i.e., file systems)
             paths = [
                 os.path.join(dp, f)
                 for dp, dn, fn in os.walk(os.path.expanduser(full_path))
                 for f in fn
             ]
             for path in paths:
-                all_paths.append(path)
-            return all_paths
+                if resource.load_folders:
+                    file_path_parts = path.split("/")
+                    folder_parts = file_path_parts[0:-1]
+                    folder = "/".join(folder_parts)
+                    all_paths.add(folder)
+                else:
+                    all_paths.add(path)
+            return list(all_paths)
 
     def get_all_data_sources(
         self, base_path: str = "catalog"
@@ -311,3 +327,71 @@ class Catalog:
         else:
             self.logger.info("Returning cached data sources")
             return self._all_data_sources
+
+    # noinspection PyMethodMayBeStatic
+    def get_last_processed_dates_for_resources(
+        self,
+        data_source: DataSource,
+        environment: HelixEnvironment,
+        resource_name: Optional[str] = None,
+    ) -> Dict[str, Optional[datetime]]:
+        """
+        Get the last processed dates of all the resources in this data source
+
+
+        :param data_source: data source to get
+        :param resource_name: (Optional) limit results to this resource
+        :param environment: environment
+        """
+        resource: ResourceItem
+        resources = list(
+            filter(
+                lambda r: r.last_processed is not None
+                and (not resource_name or r.name == resource_name),
+                data_source.resources,
+            )
+        )
+        if len(resources) == 0:
+            return {}
+
+        result: Dict[str, Optional[datetime]] = {}
+        for resource in resources:
+            result[resource.name] = self.get_last_processed_date_for_resource(
+                resource=resource, environment=environment
+            )
+        return result
+
+    # noinspection PyMethodMayBeStatic
+    def get_last_processed_date_for_resource(
+        self, environment: HelixEnvironment, resource: ResourceItem
+    ) -> Optional[datetime]:
+        """
+        Gets last processed date for the specified resource
+
+
+        :param resource:
+        :param environment: environment
+        """
+        if resource.last_processed:
+            date_segment: Union[int, List[int]] = (
+                resource.date_segment if resource.date_segment else 0
+            )
+            date_format: str = (
+                resource.date_format if resource.date_format else "%Y%m%d"
+            )
+            last_processed_path: str = resource.last_processed.get_last_processed(
+                environment
+            )
+            last_processed_path_parts: List[str] = list(
+                filter(None, last_processed_path.replace("s3:/", "").split("/"))
+            )
+            if isinstance(date_segment, int):
+                return datetime.strptime(
+                    last_processed_path_parts[date_segment], date_format
+                )
+            elif isinstance(date_segment, list):
+                date_text = "".join(
+                    [last_processed_path_parts[index] for index in date_segment]
+                )
+                return datetime.strptime(date_text, date_format)
+        return None
