@@ -11,6 +11,7 @@ class ResourceItem:
         "name",
         "path",
         "path_slug",
+        "load_folders",
         "last_processed",
         "date_segment",
         "date_format",
@@ -38,6 +39,10 @@ class ResourceItem:
         self.path_extension = None
         if hasattr(content, "path_extension") and content.path_extension is not None:
             self.path_extension = content.path_extension
+        if hasattr(content, "load_folders") and content.load_folders is not None:
+            self.load_folders: bool = content.load_folders
+        else:
+            self.load_folders = False
 
     @property
     def full_path_slug(self) -> Optional[str]:
