@@ -3,8 +3,10 @@ import json
 import os.path
 import shutil
 from types import ModuleType
+from typing import Dict, Optional, List
 
 from helix_catalog_sdk.catalog import Catalog
+from helix_catalog_sdk.data_source import LastProcessed
 from helix_catalog_sdk.repo import DirectoryRepo
 from helix_catalog_sdk.enums import HelixEnvironment
 
@@ -160,6 +162,7 @@ def test_catalog_update_data_source_resource() -> None:
     json_resources = json_data.get("resources")
     assert json_resources == [
         {
+            "load_folders": False,
             "last_processed": {
                 "dev": "s3://dev-ingestion/raw/gencon/icd/People/PeopleFeed_06092021.json",
                 "production": "s3://prod-ingestion/raw/gencon/icd/People/PeopleFeed_06092021.json",
@@ -173,6 +176,7 @@ def test_catalog_update_data_source_resource() -> None:
             "date_format": None,
         },
         {
+            "load_folders": False,
             "last_processed": None,
             "name": "places",
             "path": "Places/PlaceFeed_06172021.json",
@@ -181,6 +185,7 @@ def test_catalog_update_data_source_resource() -> None:
             "date_format": None,
         },
         {
+            "load_folders": False,
             "last_processed": None,
             "name": "things",
             "path": "Things/ThingFeed_06092021.json",
@@ -191,7 +196,7 @@ def test_catalog_update_data_source_resource() -> None:
     ]
 
 
-def test_catalog_udpate_data_source_resource_with_extension() -> None:
+def test_catalog_update_data_source_resource_with_extension() -> None:
     clean_test_folder()
     repo = DirectoryRepo(location="")
     catalog = Catalog(repo=repo)
@@ -367,3 +372,98 @@ def test_catalog_get_resource_unprocessed_directories_segments() -> None:
         "data/events/2095895/2021/10/02/some-patient-file.csv",
         "data/events/2095895/2021/10/03/some-patient-file.csv",
     ]
+
+
+def test_catalog_get_resource_last_processed_date_for_single_segment() -> None:
+    clean_test_folder()
+    repo = DirectoryRepo(location="")
+    catalog = Catalog(repo=repo)
+    data_source = catalog.get_data_source(
+        "catalog/raw/thedacare/betteru.json", HelixEnvironment.PRODUCTION
+    )
+    assert data_source is not None
+    catalog.update_resource_last_processed_date_segment(
+        "betteru",
+        datetime.datetime.strptime("2021-11-11", "%Y-%m-%d"),
+        data_source,
+        HelixEnvironment.PRODUCTION,
+    )
+    data_source = catalog.get_data_source(
+        "catalog/raw/thedacare/betteru.json", HelixEnvironment.PRODUCTION
+    )
+    assert data_source is not None
+    people_resource = data_source.resources[0]
+    assert people_resource is not None
+    last_processed: Optional[LastProcessed] = people_resource.last_processed
+    assert last_processed is not None
+    assert (
+        last_processed.get_last_processed(HelixEnvironment.PRODUCTION)
+        == "data/raw/thedacare/betteru/2021-11-11/patient-files/some-patient-file.csv"
+    )
+
+    result: Dict[
+        str, Optional[datetime.datetime]
+    ] = catalog.get_last_processed_dates_for_resources(
+        data_source=data_source, environment=HelixEnvironment.PRODUCTION
+    )
+    assert result[people_resource.name] == datetime.datetime(2021, 11, 11)
+
+
+def test_catalog_get_resource_last_processed_date_for_multiple_segments() -> None:
+    clean_test_folder()
+    repo = DirectoryRepo(location="")
+    catalog = Catalog(repo=repo)
+    data_source = catalog.get_data_source(
+        "catalog/events/mixpanel/events.json", HelixEnvironment.PRODUCTION
+    )
+    assert data_source is not None
+    catalog.update_resource_last_processed_date_segment(
+        "2095895",
+        datetime.datetime.strptime("2021-11-11", "%Y-%m-%d"),
+        data_source,
+        HelixEnvironment.PRODUCTION,
+    )
+    data_source = catalog.get_data_source(
+        "catalog/events/mixpanel/events.json", HelixEnvironment.PRODUCTION
+    )
+    assert data_source is not None
+    people_resource = data_source.resources[0]
+    assert people_resource is not None
+    last_processed = people_resource.last_processed
+    assert last_processed is not None
+    assert (
+        last_processed.get_last_processed(HelixEnvironment.PRODUCTION)
+        == "data/events/2095895/2021/11/11/full_day/some-patient-file.csv"
+    )
+
+    result: Dict[
+        str, Optional[datetime.datetime]
+    ] = catalog.get_last_processed_dates_for_resources(
+        data_source=data_source, environment=HelixEnvironment.PRODUCTION
+    )
+    assert result[people_resource.name] == datetime.datetime(2021, 11, 11)
+
+
+def test_catalog_get_resource_last_processed_date_for_folders() -> None:
+    clean_test_folder()
+    repo = DirectoryRepo(location="")
+    catalog = Catalog(repo=repo)
+    data_source = catalog.get_data_source(
+        "catalog/events/mixpanel/events_parquet.json", HelixEnvironment.PRODUCTION
+    )
+    assert data_source is not None
+    assert data_source is not None
+    people_resource = data_source.resources[0]
+    assert people_resource is not None
+    last_processed = people_resource.last_processed
+    assert last_processed is not None
+    result: List[str] = catalog.get_resource_unprocessed_directories(
+        people_resource.name,
+        data_source=data_source,
+        environment=HelixEnvironment.PRODUCTION,
+    )
+
+    assert result[0] == "data/events/2095895/2021/10/02"
+    assert result[1] == "data/events/2095895/2021/10/03"
+
+    assert len(result) == 2
